-- FADD_S
-- ソースオペランドの符合は一致しているものとする

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FADD_S is

	port(SRC1,SRC2 : in std_logic_vector(31 downto 0);
	     OUTPUT    : out std_logic_vector(30 downto 0);
	     ZeroOut   : out std_logic);

end FADD_S;

architecture STRUCTURE of FADD_S is
component RightShifter24
	port(Data    : in std_logic_vector(23 downto 0);
	     nShift  : in std_logic_vector(7 downto 0);
	     Shifted : out std_logic_vector(25 downto 0));
end component;

signal Zero : std_logic;                      -- Zero

signal W : std_logic_vector(31 downto 0);     -- W数
signal L : std_logic_vector(31 downto 0);     -- L数
signal nShift : std_logic_vector(7 downto 0); -- input to RightShifter
signal EXP : std_logic_vector(7 downto 0);
signal EXP_buf : std_logic_vector(7 downto 0);
signal L_MANT_Shifted : std_logic_vector(25 downto 0); -- L数の仮数部を右シフトしたもの
signal MANT : std_logic_vector(22 downto 0);           -- MANT_buf2のMSBを左端にそろえたもの
signal MANT_buf : std_logic_vector(25 downto 0);
signal MANT_buf2 : std_logic_vector(25 downto 0);      -- MANT_bufを丸め
signal MANTb : std_logic_vector(23 downto 0);

begin -- STRUCTURER
   Zero <= '1' when (SRC1(30 downto 23) = 0 and SRC2(30 downto 23) = 0)
           else
           '0';
    
   L <= SRC1 when SRC1(30 downto 23) < SRC2(30 downto 23) else SRC2;
   W <= SRC2 when SRC1(30 downto 23) < SRC2(30 downto 23) else SRC1;


   EXP_buf <= W(30 downto 23); 
   nShift <= W(30 downto 23) - L(30 downto 23); -- RightShifter24の入力
   
   L_MANT_Shifted <= shr('1' & L(22 downto 0) & "00",nShift);
   
   MANT_buf <= ("01" & W(22 downto 0) & '0') + ('0' & L_MANT_Shifted(25 downto 1));
   EXP <= EXP_buf + 1 when (MANT_buf(25) = '1' or MANTb = "111111111111111111111111") else
          EXP_buf;
   MANT_buf2 <= MANT_buf; -- 丸め
   MANTb <= MANT_buf2(24 downto 1) when MANT_buf2(25) = '1' else
            MANT_buf2(23 downto 0);
           
   MANT <= MANTb(23 downto 1) + MANTb(0);
   
   ZeroOut <= Zero;
   
   OUTPUT <= (others => '0') when Zero = '1' else
             EXP & MANT;
end STRUCTURE;
