-- FMUL

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FMUL is

	port(SRC1,SRC2 : in std_logic_vector(31 downto 0);
	     OUTPUT    : out std_logic_vector(31 downto 0));

end FMUL;

architecture STRUCTURE of FMUL is
signal Zero : std_logic;                      -- Zero
signal S : std_logic;
signal EXP_buf2 : std_logic_vector(7 downto 0); -- SRC1+SRC2
--signal EXP : std_logic_vector(7 downto 0);
signal MANT_buf : std_logic_vector(25 downto 0); -- SRC1,SRC2の仮数部をそのまま掛けたもの
signal MANT_buf3 : std_logic_vector(23 downto 0); -- LeftShift後
--signal MANT_buf4 : std_logic_vector(22 downto 0); -- 丸め後

signal SMANT1_high : std_logic_vector(6 downto 0); --24bit×24bitの分割に使用
signal SMANT1_low  : std_logic_vector(16 downto 0);
signal SMANT2_high : std_logic_vector(6 downto 0);
signal SMANT2_low  : std_logic_vector(16 downto 0);
signal SMANT_hh : std_logic_vector(13 downto 0); -- SMANT1_high × SMANT2_high
signal SMANT_hl : std_logic_vector(23 downto 0);
signal SMANT_lh : std_logic_vector(23 downto 0);
signal SMANT_ll : std_logic_vector(33 downto 0);
signal EXPMANT : std_logic_vector(30 downto 0);
--signal carry : std_logic;

begin -- STRUCTURE
   Zero <= '1' when SRC1(30 downto 23) = 0 or SRC2(30 downto 23) = 0 else
           '0';
   S <= SRC1(31) xor SRC2(31); -- 符号
   EXP_buf2 <= SRC1(30 downto 23) + SRC2(30 downto 23);
   
   SMANT1_high <= '1' & SRC1(22 downto 17);
   SMANT1_low  <= SRC1(16 downto 0);
   SMANT2_high <= '1' & SRC2(22 downto 17);
   SMANT2_low  <= SRC2(16 downto 0);
   SMANT_hh <= (SMANT1_high * SMANT2_high);
   SMANT_hl <= SMANT1_high * SMANT2_low;
   SMANT_lh <= SMANT1_low * SMANT2_high;
   SMANT_ll <= (SMANT1_low * SMANT2_low);
   MANT_buf <= (SMANT_hh & SMANT_ll(33 downto 22)) + ("000000" & (('0' & SMANT_hl(23 downto 5)) + ('0' & SMANT_lh(23 downto 5))));

   MANT_buf3 <= MANT_buf(24 downto 1) when MANT_buf(25) = '1' else -- 桁揃え
                MANT_buf(23 downto 0);-- when MANT_buf(25) = '0';
   --MANT_buf4 <= MANT_buf3(23 downto 1) + MANT_buf3(0); -- 丸め

   EXPMANT <= ((EXP_buf2 - "01111110") & MANT_buf3(23 downto 1)) + MANT_buf3(0) when MANT_buf(25) = '1' else
              ((EXP_buf2 - "01111111") & MANT_buf3(23 downto 1)) + MANT_buf3(0);

   OUTPUT <= "00000000000000000000000000000000" when Zero = '1' else
             S & EXPMANT;
end STRUCTURE;
